from django.middleware.csrf import get_token
from django.template.loader import render_to_string
from django.http.response import HttpResponse, HttpResponseRedirect
from django.views.generic.base import (ContextMixin, View,
                                       TemplateResponseMixin)

from django_modalview.generic.component import (ModalButton, GET_TEMPLATE,
                                                GET_TEMPLATE_CONTENT,
                                                BASE_TEMPLATE)
from django_modalview.generic.response import (ModalJsonResponse,
                                               ModalJsonResponseRedirect)


class ModalContextMixin(ContextMixin):

    """
            A default modal context mixin that passes the keyword arguments
            received by get_context_modal_data as the modal template context.
    """

    def __init__(self, title=None, description=None, icon=None, *args,
                 **kwargs):
        super(ContextMixin, self).__init__(*args, **kwargs)
        self.title = "title"
        self.response = None
        self.icon = icon
        self.description = ""
        self.close_button = ModalButton('Close', button_type='primary')
        self.content_template_name = None
        self.base_template_name = BASE_TEMPLATE
        # use to know if you can redirect. Disable for the first request.

    def _generate_modal_context(self):
        return {
            'title': self.title,
            'description': self.description,
            'button_close': self.close_button,
            'content_template_name': self.content_template_name,
            'base_template_name': self.base_template_name,
            'icon': self.icon,
            'response': self.response,
        }

    def get_context_modal_data(self, **kwargs):
        kwargs.update(self._generate_modal_context())
        return kwargs


class ModalView(View):

    """
            Parent class of all the ModalView. Extends the Django generic View
            to override the dispatch method and to overload the get method.
    """

    def __init__(self, *args, **kwargs):
        super(ModalView, self).__init__(*args, **kwargs)
        self.is_ajax = False
        self._can_redirect = False
        self.redirect_to = None

    def can_redirect(self):
        return self._can_redirect and self.redirect_to

    def dispatch(self, request, *args, **kwargs):
        self.is_ajax = request.is_ajax()
        return super(ModalView, self).dispatch(request, *args, **kwargs)

    def get(self, request, *args, **kwargs):
        context = self.get_context_modal_data(**kwargs)
        return self.render_to_response(
            context=context
        )


class ModalTemplateMixin(TemplateResponseMixin):

    """
            Mixin use to render a template. A modal view can be called by a
            simple Http request or by an Ajax request. The type of the response
            is different for these two cases.
    """
    json_response_class = ModalJsonResponse
    json_response_redirect_class = ModalJsonResponseRedirect
    http_response_class = HttpResponse
    http_response_redirect_class = HttpResponseRedirect

    def _valid_template(self):
        if not self.is_ajax:
            self.template_name = GET_TEMPLATE

    def _get_content(self, context):
        """
                Add the csrf_token_value because the mixin use render_to_string
                and not render.
        """
        self._valid_template()
        context.update({
            "csrf_token_value": get_token(self.request)
        })
        return render_to_string(self.get_template_names(), context)

    def get_response(self):

        if self.is_ajax:
            if not self.can_redirect():
                return ModalTemplateMixin.json_response_class
            else:
                return ModalTemplateMixin.json_response_redirect_class
        else:
            if not self.can_redirect():
                return ModalTemplateMixin.http_response_class
            else:
                return ModalTemplateMixin.http_response_redirect_class

    def render_to_response(self, context):
        ResponseClass = self.get_response()
        if not self.can_redirect():
            context.update(self.get_context_data())
            return ResponseClass(self._get_content(context))
        else:
            return ResponseClass(self.redirect_to)


class ModalUtilMixin(object):

    """
            Mixin that permit to launch a method with a modal button.
    """

    def __init__(self, *args, **kwargs):
        super(ModalUtilMixin, self).__init__(*args, **kwargs)
        self.util_kwargs = {}

    def get_util(self, func_name, *args, **kwargs):
        if hasattr(self, func_name):
            util_kwargs = self.get_util_kwargs(**kwargs)
            getattr(self, func_name)(*args, **util_kwargs)
        else:
            raise Exception("You should implement one method name"
                            " {name}!".format(name=func_name))

    def get_util_kwargs(self, *args, **kwargs):
        kwargs.update(self.request.GET)
        self.util_kwargs.update(**kwargs)
        return self.util_kwargs


class BaseModalView(ModalContextMixin, ModalView):

    """
            A base view to handle a simple modal
    """

    def __init__(self, *args, **kwargs):
        super(BaseModalView, self).__init__(*args, **kwargs)
        self.template_name = GET_TEMPLATE
        self.content_template_name = GET_TEMPLATE_CONTENT


class ModalTemplateView(ModalTemplateMixin, BaseModalView):

    """
            A view that display a simple modal
    """


class ModalTemplateUtilView(ModalUtilMixin, ModalTemplateView):

    """
            A view that display a modal and that is able to handle an util method.
            A new button is displayed in the modal to run the tool.
    """

    def __init__(self, button=None, *args, **kwargs):
        super(ModalTemplateUtilView, self).__init__(*args, **kwargs)
        self.util_button = button if button else ModalButton('Run test')
        self.util_name = 'util'

    def dispatch(self, request, *args, **kwargs):
        self.util_button.url = request.META['PATH_INFO'] + '?util=true'
        return super(ModalTemplateUtilView, self).dispatch(request, *args,
                                                           **kwargs)

    def get_context_modal_data(self, **kwargs):
        kwargs.update({
            'util_button': self.util_button
        })
        return super(ModalTemplateUtilView,
                     self).get_context_modal_data(**kwargs)

    def get(self, request, *args, **kwargs):
        get_dict = self.request.GET
        if get_dict.get('util'):
            self._can_redirect = True
            self.get_util(self.util_name, **self.kwargs)
            self.template_name = self.content_template_name

        return super(ModalTemplateUtilView, self).get(request, *args,
                                                      **kwargs)



import json

from django.http.response import HttpResponse
from django.core.exceptions import ImproperlyConfigured


class BaseModalJsonResponse(HttpResponse):
    content_type = 'application/json'

    def __init__(self, *args, **kwargs):
        jsn_content = json.dumps(self.get_content())
        super(BaseModalJsonResponse, self).__init__(jsn_content,
                                                    self.content_type,
                                                    *args, **kwargs)

    def get_content(self):
        raise ImproperlyConfigured("No method get_content")


class ModalJsonResponse(BaseModalJsonResponse):

    type = 'normal'

    def __init__(self, content, *args, **kwargs):
        self.modal_content = content
        super(ModalJsonResponse, self).__init__(*args, **kwargs)

    def get_content(self):
        return {
            'type': self.type,
            'content': self.modal_content
        }


class ModalJsonResponseRedirect(BaseModalJsonResponse):
    type = 'redirect'

    def __init__(self, redirect_to, *args, **kwargs):
        self.redirect_to = redirect_to
        super(ModalJsonResponseRedirect, self).__init__(*args, **kwargs)

    def get_content(self):
        return {
            'type': self.type,
            'redirect_to': self.redirect_to
        }



GET_TEMPLATE = "django_modalview/modal.html"
GET_TEMPLATE_CONTENT = "django_modalview/modal_get_content.html"
FORM_TEMPLATE = "django_modalview/modal.html"
FORM_TEMPLATE_CONTENT = "django_modalview/modal_form_content.html"
LAST_FORM_TEMPLATE = "django_modalview/form_content.html"
BASE_TEMPLATE = "django_modalview/base.html"


class ModalButton(object):

    def __init__(self, value=None, button_type='info',
                 display=True, url=None, *args, **kwargs):

        super(ModalButton, self).__init__(*args, **kwargs)
        self.value = value
        self.display = display
        self.type = button_type
        self.url = url


class ModalResponse(object):

    def __init__(self, text='Result', result='info'):
        self.text = text
        self.result = result



from django.views.generic.edit import (FormMixin, ProcessFormView,
                                       ModelFormMixin, DeletionMixin)

from django_modalview.generic.base import (ModalContextMixin, ModalView,
                                           ModalTemplateMixin, ModalUtilMixin)
from django_modalview.generic.component import (FORM_TEMPLATE_CONTENT,
                                                LAST_FORM_TEMPLATE,
                                                FORM_TEMPLATE, ModalButton)


class ModalEditContextMixin(ModalContextMixin):

    """
            Mixin that extends the ModalContextMixin to add some new elements in
            the modal context.

    """

    def __init__(self, *args, **kwargs):
        super(ModalEditContextMixin, self).__init__(*args, **kwargs)
        self.action = None
        self.content_template_name = FORM_TEMPLATE_CONTENT
        self.submit_button = ModalButton(value='send', button_type='primary')
        self.form_content_template_name = LAST_FORM_TEMPLATE

    def get_context_modal_data(self, **kwargs):
        kwargs.update({
            'submit_button': self.submit_button,
            'action': self.action,
            'form_template_name': self.form_content_template_name
        })
        return super(ModalEditContextMixin,
                     self).get_context_modal_data(**kwargs)


class ModalFormMixin(ModalEditContextMixin, FormMixin):

    """
            Mixin that provide a way to show and to handle a modal with a django
            form.
    """

    def get_context_modal_data(self, **kwargs):
        kwargs.update({
            'form': self.get_form(self.get_form_class()),
        })
        return super(ModalFormMixin, self).get_context_modal_data(**kwargs)

    def _form_response(self, **kwargs):
        kwargs.update(self.get_context_modal_data())
        return self.render_to_response(context=kwargs)

    def form_valid(self, form, **kwargs):
        self._can_redirect = True
        return self._form_response(**kwargs)

    def form_invalid(self, form, **kwargs):
        return self._form_response(**kwargs)


class ModalFormUtilMixin(ModalFormMixin, ModalUtilMixin):
    """
        Mixin that provide a way to show and to handle a modal with a django
        form
    """
    def util_on_form_valid(self, **kwargs):
        pass

    def util_on_form_invalid(self, **kwargs):
        pass

    def dispatch(self, request, *args, **kwargs):
        self.kwargs.update(kwargs)
        self.kwargs.update(request.GET)
        return super(ModalFormUtilMixin, self).dispatch(request, *args,
                                                        **kwargs)

    def form_valid(self, form, **kwargs):
        self.kwargs.update(**form.cleaned_data)
        self.get_util('util_on_form_valid', **self.kwargs)
        return super(ModalFormUtilMixin, self).form_valid(form, **kwargs)

    def form_invalid(self, form, **kwargs):
        self.kwargs.update(**form.cleaned_data)
        self.get_util('util_on_form_invalid', **self.kwargs)
        return super(ModalFormUtilMixin, self).form_invalid(form, **kwargs)


class ModalModelFormMixin(ModalFormMixin, ModelFormMixin):

    """
            A mixin that provide a way to show and to handle a modelform in a
            modal
    """

    def save(self, form):
        self.object = form.save()

    def form_valid(self, form, commit=True, **kwargs):
        if commit:
            self.save(form)
        return super(ModalModelFormMixin, self).form_valid(form, **kwargs)


class BaseProcessModalView(ModalView, ProcessFormView):

    """
            A base mixin to handle a get request on a modal view that contains
            a form.
    """

    def dispatch(self, request, *args, **kwargs):
        self.action = request.path
        return super(BaseProcessModalView, self).dispatch(request, *args,
                                                          **kwargs)

    def get(self, request, *args, **kwargs):
        self.template_name = FORM_TEMPLATE
        return super(BaseProcessModalView, self).get(request, *args, **kwargs)


class ProcessModalFormView(BaseProcessModalView):

    """
            A mixin that provide a way to handle a modal view with a form on a
            GET request and to run it on a POST request.
    """

    def post(self, request, *args, **kwargs):
        self.template_name = self.content_template_name
        return super(ProcessModalFormView, self).post(request, *args,
                                                      **kwargs)


class ProcessModalPostView(BaseProcessModalView):

    """
            A mixin that provide a way to handle a modal view with a form on a
            GET request. The post request don't check if the form is valid. Usefull
            for the deletion Mixin or other modal that want to run an util on a
            POST request without analize the form
    """

    def post(self, request, *args, **kwargs):
        self.template_name = self.content_template_name
        self._can_redirect = True
        kwargs.update(self.get_context_modal_data())
        return self.render_to_response(context=kwargs)


class ModalDeletionMixin(ModalEditContextMixin):

    """
            A mixin that provide a way to delete an object.
    """

    def delete(self, request, *args, **kwargs):
        self.object.delete()


class ModalPostMixin(ModalEditContextMixin):
    '''
        A mixin that provide a way to handle a post request
    '''


class ModalPostUtilMixin(ModalPostMixin, ModalUtilMixin):

    def util_on_post(self, *args, **kwargs):
        pass

    def dispatch(self, request, *args, **kwargs):
        self.kwargs.update(kwargs)
        self.kwargs.update(request.GET)
        return super(ModalPostUtilMixin, self).dispatch(request, *args,
                                                        **kwargs)

    def post(self, request, *args, **kwargs):
        self.kwargs.update(kwargs)
        self.get_util('util_on_post', **self.kwargs)
        return super(ModalPostUtilMixin, self).post(request, **kwargs)


class BaseModalUpdateView(ModalModelFormMixin, ProcessModalFormView):

    """
            A base view that provide a way to display a modelform, in a modal,
            to update an object. The attribute that contains the instance is
            self.object.
    """


class BaseModalCreateView(BaseModalUpdateView):

    """
            A base view that provide a way to display a modelform, in a modal,
            to update an object. The attribute that contains the instance is
            self.object.
    """

    def get(self, request, *args, **kwargs):
        self.object = None
        return super(BaseModalCreateView, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.object = None
        return super(BaseModalCreateView, self).post(request, *args, **kwargs)


class BaseModalDeleteView(ModalDeletionMixin, ProcessModalPostView):

    """
            A base view that provide a way to delete an object in a modal.
            The object that sould be deleted is in the attribute self.object
    """

    def __init__(self, *args, **kwargs):
        super(BaseModalDeleteView, self).__init__(*args, **kwargs)
        self.submit_button.type = 'danger'
        self.submit_button.value = 'Delete'

    def post(self, request, *args, **kwargs):
        self.delete(request, *args, **kwargs)
        return super(BaseModalDeleteView, self).post(request, *args, **kwargs)


class BaseModalPostView(ModalPostMixin, ProcessModalPostView):

    """
            A base view that provide a way to handle a modal that can send a post
            request and handle its response.
    """


class BaseModalPostUtilView(ModalPostUtilMixin, ProcessModalPostView):
    """
            A base view that provide a way to handle a modal that can send a post
            request, run an util and handle its response.
    """


class BaseModalFormView(ModalFormMixin, ProcessModalFormView):

    """
            A base view that provide a way to handle a modal with a form.
    """


class BaseModalFormUtilView(ModalFormUtilMixin, ProcessModalFormView):

    """
        A base view that provide a way to handle a modal with form and util
    """


class ModalFormView(ModalTemplateMixin, BaseModalFormView):

    """
            A view to handle a modal with form and to render it.
    """


class ModalFormUtilView(ModalTemplateMixin, BaseModalFormUtilView):

    """
        A view to handle a modal with form/util and to render it.
    """


class ModalCreateView(ModalTemplateMixin, BaseModalCreateView):

    """
            A view to handle a modal with a create form and to render it.
    """


class ModalUpdateView(ModalTemplateMixin, BaseModalUpdateView):

    """
            A view to handle a modal with an update form and to render it.
    """


class ModalPostView(ModalTemplateMixin, BaseModalPostView):

    """
            A view to handle a modal with a post request sender and to render it.
    """


class ModalPostUtilView(ModalTemplateMixin, BaseModalPostUtilView):
    """
    """


class ModalDeleteView(ModalTemplateMixin, BaseModalDeleteView):

    """
            A view to handle a modal with delete button and to render it.
    """
